@@ -12,15 +12,10 @@ my $resolver = $loop->resolver;
 $resolver->res_query(
    dname => $ARGV[0],
    type  => "PTR",
-   on_resolved => sub {
-      my ( $pkt, @names ) = @_;
+)->on_done( sub {
+   my ( $pkt, @names ) = @_;
 
-      foreach my $name ( @names ) {
-         print "$name\n";
-      }
-      $loop->stop;
-   },
-   on_error => sub { die "Cannot resolve - $_[-1]" },
-);
-
-$loop->run;
+   foreach my $name ( @names ) {
+      print "$name\n";
+   }
+})->get;
