@@ -1,16 +1,17 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2015 -- leonerd@leonerd.org.uk
 
 package IO::Async::Resolver::DNS;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
-use IO::Async::Resolver;
+use Future;
+use IO::Async::Resolver 0.52; # returns Future
 
 use Carp;
 use Net::DNS;
@@ -25,28 +26,23 @@ C<IO::Async::Resolver::DNS> - resolve DNS queries using C<IO::Async>
 
  use IO::Async::Loop;
  use IO::Async::Resolver::DNS;
- 
+
  my $loop = IO::Async::Loop->new;
  my $resolver = $loop->resolver;
- 
+
  $resolver->res_query(
     dname => "cpan.org",
     type  => "MX",
-    on_resolved => sub {
-       my ( $pkt ) = @_;
- 
-       foreach my $mx ( $pkt->answer ) {
-          next unless $mx->type eq "MX";
- 
-          printf "preference=%d exchange=%s\n",
-             $mx->preference, $mx->exchange;
-       }
-       $loop->stop;
-    },
-    on_error => sub { die "Cannot resolve - $_[-1]\n" },
- );
- 
- $loop->run;
+ )->then( sub {
+    my ( $pkt ) = @_;
+
+    foreach my $mx ( $pkt->answer ) {
+       next unless $mx->type eq "MX";
+
+       printf "preference=%d exchange=%s\n",
+          $mx->preference, $mx->exchange;
+    }
+ })->get;
 
 =head1 DESCRIPTION
 
@@ -73,7 +69,9 @@ other situations where C<Net::LibResolv> is not available.
 If certain record type queries are made, extra information is returned to the
 C<on_resolved> continuation, containing the results from the DNS packet in a
 more useful form. This information will be in a list of extra values following
-the packet value
+the packet value.
+
+ my ( $pkt, @data ) = $f->get;
 
  $on_resolved->( $pkt, @data )
 
@@ -228,9 +226,14 @@ sub _extract_SRV
 
 =head1 RESOLVER METHODS
 
+The following methods documented with a trailing call to C<< ->get >> return
+L<Future> instances.
+
 =cut
 
-=head2 $resolver->res_query( %params )
+=head2 res_query
+
+   ( $pkt, @data ) = $resolver->res_query( %params )->get
 
 Performs a resolver query on the name, class and type, and invokes a
 continuation when a result is obtained.
@@ -252,6 +255,13 @@ Name of the record type to look up (e.g. C<MX>)
 Name of the record class to look up. Defaults to C<IN> so normally this
 argument is not required.
 
+=back
+
+When not returning a C<Future>, the following extra arguments are used as
+callbacks instead:
+
+=over 8
+
 =item on_resolved => CODE
 
 Continuation which is invoked after a successful lookup. Will be passed a
@@ -283,22 +293,27 @@ sub IO::Async::Resolver::res_query
    my $class = $args{class} || "IN";
    my $type  = $args{type}  or croak "Expected 'type'";
 
-   my $on_resolved = $args{on_resolved};
-   ref $on_resolved or croak "Expected 'on_resolved' to be a reference";
+   my $on_resolved = delete $args{on_resolved};
+   !$on_resolved or ref $on_resolved or croak "Expected 'on_resolved' to be a reference";
 
-   $self->resolve(
+   my $f = $self->resolve(
       type => "res_query",
       data => [ $dname, $class, $type ],
-      on_resolved => sub {
-         my ( $data ) = @_;
-         my $pkt = Net::DNS::Packet->new( \$data );
-         $on_resolved->( _extract( $pkt, $type ) );
-      },
-      on_error => $args{on_error},
-   );
+   )->then( sub {
+      my ( $data ) = @_;
+      my $pkt = Net::DNS::Packet->new( \$data );
+      Future->done( _extract( $pkt, $type ) );
+   });
+
+   $f->on_done( $on_resolved ) if $on_resolved;
+   $f->on_fail( $args{on_error} ) if $args{on_error};
+
+   $self->adopt_future( $f ) unless defined wantarray;
+
+   return $f;
 }
 
-=head2 $resolver->res_search( %params )
+=head2 res_search
 
 Performs a resolver query on the name, class and type, and invokes a
 continuation when a result is obtained. Identical to C<res_query> except that
@@ -315,19 +330,24 @@ sub IO::Async::Resolver::res_search
    my $class = $args{class} || "IN";
    my $type  = $args{type}  or croak "Expected 'type'";
 
-   my $on_resolved = $args{on_resolved};
-   ref $on_resolved or croak "Expected 'on_resolved' to be a reference";
+   my $on_resolved = delete $args{on_resolved};
+   !$on_resolved or ref $on_resolved or croak "Expected 'on_resolved' to be a reference";
 
-   $self->resolve(
+   my $f = $self->resolve(
       type => "res_search",
       data => [ $dname, $class, $type ],
-      on_resolved => sub {
-         my ( $data ) = @_;
-         my $pkt = Net::DNS::Packet->new( \$data );
-         $on_resolved->( _extract( $pkt, $type ) );
-      },
-      on_error => $args{on_error},
-   );
+   )->then( sub {
+      my ( $data ) = @_;
+      my $pkt = Net::DNS::Packet->new( \$data );
+      Future->done( _extract( $pkt, $type ) );
+   });
+
+   $f->on_done( $on_resolved ) if $on_resolved;
+   $f->on_fail( $args{on_error} ) if $args{on_error};
+
+   $self->adopt_future( $f ) unless defined wantarray;
+
+   return $f;
 }
 
 # We'd prefer to use libresolv to actually talk DNS as it'll be more efficient
